@@ -6,7 +6,7 @@ Tests to make sure the behaviour of the builtins is sensible and correct.
 from __future__ import absolute_import, division, print_function, unicode_literals
 from future.builtins import (bytes, dict, int, range, round, str, super,
                              ascii, chr, hex, input, next, oct, open, pow,
-                             filter, map, zip)
+                             filter, map, zip, min, max)
 
 from future.utils import PY3, exec_, native_str, implements_iterator
 from future.tests.base import (unittest, skip26, expectedFailurePY2,
@@ -146,7 +146,6 @@ class TestBuiltins(unittest.TestCase):
         self.assertTrue(isinstance(round(123.5, 0), float))
         self.assertTrue(isinstance(round(123.5), Integral))
 
-    @unittest.skip('negative ndigits not implemented yet')
     def test_round_negative_ndigits(self):
         self.assertEqual(round(10.1350, 0), 10.0)
         self.assertEqual(round(10.1350, -1), 10.0)
@@ -524,11 +523,11 @@ class BuiltinTest(unittest.TestCase):
         self.assertRaises(TypeError, compile)
         self.assertRaises(ValueError, compile, 'print(42)\n', '<string>', 'badmode')
         self.assertRaises(ValueError, compile, 'print(42)\n', '<string>', 'single', 0xff)
-        self.assertRaises(TypeError, compile, chr(0), 'f', 'exec')
+        # Raises TypeError in Python < v3.5, ValueError in v3.5:
+        self.assertRaises((TypeError, ValueError), compile, chr(0), 'f', 'exec')
         self.assertRaises(TypeError, compile, 'pass', '?', 'exec',
                           mode='eval', source='0', filename='tmp')
         compile('print("\xe5")\n', '', 'exec')
-        self.assertRaises(TypeError, compile, chr(0), 'f', 'exec')
         self.assertRaises(ValueError, compile, str('a = 1'), 'f', 'bad')
 
         # test the optimize argument
@@ -1099,6 +1098,14 @@ class BuiltinTest(unittest.TestCase):
         self.assertEqual(max(data, key=f),
                          sorted(reversed(data), key=f)[-1])
 
+        self.assertEqual(max([], default=5), 5)
+        with self.assertRaises(TypeError):
+            max(None, default=5)
+        with self.assertRaises(TypeError):
+            max(1, 2, default=0)
+        self.assertEqual(max([], default=0), 0)
+        self.assertIs(max([], default=None), None)
+
     def test_min(self):
         self.assertEqual(min('123123'), '1')
         self.assertEqual(min(1, 2, 3), 1)
@@ -1116,6 +1123,7 @@ class BuiltinTest(unittest.TestCase):
             def __getitem__(self, index):
                 raise ValueError
         self.assertRaises(ValueError, min, BadSeq())
+        self.assertEqual(max(x for x in [5, 4, 3]), 5)
 
         for stmt in (
             "min(key=int)",                 # no args
@@ -1140,6 +1148,16 @@ class BuiltinTest(unittest.TestCase):
         f = keys.__getitem__
         self.assertEqual(min(data, key=f),
                          sorted(data, key=f)[0])
+        self.assertEqual(min([], default=5), 5)
+        self.assertEqual(min([], default=0), 0)
+        self.assertIs(min([], default=None), None)
+        with self.assertRaises(TypeError):
+            max(None, default=5)
+        with self.assertRaises(TypeError):
+            max(1, 2, default=0)
+
+        # Test iterables that can only be looped once #510
+        self.assertEqual(min(x for x in [5]), 5)
 
     def test_next(self):
         it = iter(range(2))
@@ -1285,7 +1303,8 @@ class BuiltinTest(unittest.TestCase):
         self.assertAlmostEqual(pow(-1, 0.5), 1j)
         self.assertAlmostEqual(pow(-1, 1/3), 0.5 + 0.8660254037844386j)
 
-        self.assertRaises(TypeError, pow, -1, -2, 3)
+        # Raises TypeError in Python < v3.5, ValueError in v3.5:
+        self.assertRaises((TypeError, ValueError), pow, -1, -2, 3)
         self.assertRaises(ValueError, pow, 1, 2, 0)
 
         self.assertRaises(TypeError, pow)
